/*********************
 * object declarations
 *********************/
var photoProperties;
var photoCropper;
function OriginalData() {}


/*********************
 * constants
 *********************/
const MODE_DISPLAY_SINGLE = 0;
const MODE_DISPLAY_MULTI = 1;
const MODE_DISPLAY_NO_PHOTOS = 2;
const UPLOAD_TYPE_NORMAL = 0;
const UPLOAD_TYPE_DND = 1;

// exposed methods: setSelection, clearSelection

photoProperties = {
    mDisplayMode: null,
    mUploads: null,
    mOriginalData: null,
    mRotator: null,
    mUploadType: null,
    mInit: false,
    mValid: true,
    uploadSvc: Components.classes['@mozilla.org/rdf/datasource;1?name=flock-photo-uploads']
        .getService(Components.interfaces.flockIPhotoUploadService),
    mApiMgr: Components.classes['@flock.com/photo-api-manager;1?']
                                .getService(Components.interfaces.flockIPhotoAPIManager),
    
    invalidate: function() {
        this.mValid = false;
    },

    init: function() {
        if (this.mInit)
            return;
        this.mInit = true;
        var inst = this;
        var cropperListener = {
             onChange: function(aCropObj) {
             dump('onchange'+'\n\n\n');
                inst.updateCommands();
             },
         }
         $('image').setListener(cropperListener);

        var changeListener = {
            handleEvent: function(e) {
                dump(e.type + "\n");
                switch(e.type) {
                    case "RadioStateChange":
                    case "CheckboxStateChange":
                        $("permissions_change").checked = true;
                        break;
                }
                
            }
        }
        $("is_public").addEventListener("RadioStateChange", changeListener, false);
        $("is_friend").addEventListener("CheckboxStateChange", changeListener, false);
        $("is_family").addEventListener("CheckboxStateChange", changeListener, false);
        
        this.setupUIFields(this.mApiMgr.getDefaultService().shortName);
    },
    
    setDisplayMode: function(aDisplayMode) {
        this.mDisplayMode= aDisplayMode;
    },
    
    setUploadType: function(aUploadType) {
        this.mUploadType = aUploadType;
    },
    
    setSelection: function(aUploads) {
        this.init();
        this.mValid = true;
        // before changing to the next selection, save the previous
        if (this.mUploads!=null && this.mUploads.length) this.synch();
        this.mUploads = aUploads;
        this.mOriginalData = new OriginalData();
        this.mRotator = new PhotoRotater();
        this.setupPreviewArea();
        $('flock-photoprops-disable').setAttribute('hidden', true);
        this.updateCommands();
    },
    
    getSelection: function() {
        return this.mUploads;
    },

    updateSelectedItem: function(aUpload) {
        var upload = null;
        if(!this.mUploads) return;
        for(var i=0;i<this.mUploads.length;++i) {
            if(this.mUploads[i].id==aUpload.id) {
                this.mUploads[i] = aUpload;
                return true;
            }
        }
        return false;
    },
    
    clearSelection: function() {
        this.setDisplayMode(MODE_DISPLAY_NO_PHOTOS);
        this.mUploads = null;
        this.setupEmptyImage();
        this.updateCommands();
    },
    
    crop: function(aDegrees) {
        if(this.mUploads.length!=1) alert("crop assertion failure");
        var cropper = $("image");
        this.uploadSvc.cropPhotoUpload(this.mUploads[0], cropper.x1, cropper.y1, cropper.x2, cropper.y2);
        this.uploadSvc.savePhotoUpload(this.mUploads[0]);
        this.setSelection(this.mUploads);
    },

    rotate: function(aDegrees) {
        for (var i=0; i < this.mUploads.length; i++) {
            this.mRotator.add(this.mUploads[i], aDegrees);
        }
        this.setSelection(this.mUploads);
    },
    
    revert: function() {
        //this.setSelection(this.mUploads);
        this.uploadSvc.revertPhotoUpload(this.mUploads[0]);
        this.setSelection(this.mUploads);
    },
    
    setupPreviewArea: function() {
        if (this.mUploads.length > 1) {
            this.setDisplayMode(MODE_DISPLAY_MULTI);
            this.setupMultiImage();
        } 
        else if(this.mUploads.length==0) {
            this.clearSelection();
        }
        else {
            this.setDisplayMode(MODE_DISPLAY_SINGLE);
            this.setupSingleImage();  
        }
    }, 
    
    setupEmptyImage: function() {
        $("image").src = '';
        $("image-box").removeAttribute("collapsed");
        $("image-deck").selectedIndex = 0;
        $("title").value = '';
        $("filename").value = '';
        $("description").value = '';
        $("tags").value = '';
        $("flock-photoprops-disable").removeAttribute('hidden');
    },
    
    setupSingleImage: function() {
        var path = this.mUploads[0].previewFilePath;
        // set the background of the canvas as the preview of the image
        var spec = this.mUploads[0].previewFileSpec;
            
        $("image").src = spec;
        $("image-box").removeAttribute("collapsed");
        $("image-deck").selectedIndex = 0;
        
        $("title").value = this.mUploads[0].title;
        $("filename").value = this.mUploads[0].fileName;
		$("filename").removeAttribute('disabled');
        $("description").value = this.mUploads[0].description;
        $("tags").value = this.mUploads[0].tags;

        var is_public = this.mUploads[0].is_public;
        var is_family = this.mUploads[0].is_family;
        var is_friend = this.mUploads[0].is_friend;;

        if(is_public=="false") {
            this.setPrivacy("private");
        }
        else {
            this.setPrivacy("public");
        }

        if(is_family=="true") {
            $("is_family").checked = true;
        } 
        else {
            $("is_family").checked = false;
        }
        if(is_friend=="true") {
            $("is_friend").checked = true;
        } 
        else {
            $("is_friend").checked = false;
        }
        
        this.mOriginalData.remember();
    },
    
    setupMultiImage: function() {
        $("image-deck").selectedIndex = 1;
        $("image-box").setAttribute("collapsed", true);
        // clean up
        flock_clearChildren($("multi-image"));
        
        var title, description, tags, pub, friends, family, fileName;
        
        title = this.mUploads[0].getAttribute("title");
        fileName  = this.mUploads[0].getAttribute("fileName");
        description = this.mUploads[0].getAttribute("description");
        tags = this.mUploads[0].getAttribute("tags").split(",").sort().join(",");
        pub = this.mUploads[0].getAttribute("is_public");
        friends = this.mUploads[0].getAttribute("is_friend");
        family = this.mUploads[0].getAttribute("is_family");

        for(var i = 0; i < this.mUploads.length; ++i) {
            var newElement = document.createElement("vbox");
            newElement.appendChild(document.createElement("photopreview"));
            $("multi-image").appendChild(newElement);
            newElement.firstChild.setAttribute("src", this.mUploads[i].getAttribute('thumbFileSpec'));

            //gLocalSelection[i] = newElement;
            
            // check for common group properties
            title = this.mUploads[i].getAttribute("title") == title ? title : null;
            description = this.mUploads[i].getAttribute("description") == description ? description : null;
            tags = this.mUploads[i].getAttribute("tags").split(",").sort().join(",") == tags ? tags : null;
            pub = this.mUploads[i].getAttribute("is_public") == pub ? pub : null;
            friends = this.mUploads[i].getAttribute("is_friend") == friends ? friends : null;
            family = this.mUploads[i].getAttribute("is_family") == family ? family : null;
        }
        
        $("title").value = title;
        $("filename").value = '';
		$("filename").setAttribute('disabled', true)
        $("description").value = description;
        $("tags").value = tags;

        if (pub) {
            if (pub == "false") {
                if (friends && family) {
                    this.setPrivacy("private");
                    if (friends == "true") $('is_friend').checked = true;
                    if (family == "true") $('is_family').checked = true;
                } else {
                    $('is_friend').disabled = true;
                    $('is_family').disabled = true;
                }
            }
            if (pub == "true") {
                this.setPrivacy("public");
                $('is_friend').disabled = true;
                $('is_family').disabled = true;
            }
        } else {
            $('is_friend').disabled = true;
            $('is_family').disabled = true;
        }
        
        // update layout for group edit
        $("title").setAttribute("oninput", "$('title_change').checked = true");
        $("description").setAttribute("oninput", "$('description_change').checked = true");
        $("tags").setAttribute("oninput", "$('tags_change').checked = true");

        $("title_change").checked = false;
        $("description_change").checked = false;
        $("tags_change").checked = false;
        $("permissions_change").checked = false;
    },
    
    setPrivacy: function (aVal) {
        $('is_public').value = aVal;
        if(aVal=="private") {
            $('is_friend').disabled = false;
            $('is_family').disabled = false;
        }
        else {
            $('is_friend').disabled = true;
            $('is_family').disabled = true;
        }
    },
    
    save: function() {
        if (this.mUploadType == UPLOAD_TYPE_DND)
            return;
        if(!this.mValid) 
            return;
        for (var i=0; i < this.mUploads.length; i++) {
            try {
                dump($() + "   "  + this.mUploads[i].id + " < nodeNames\n");
                this.uploadSvc.savePhotoUpload(this.mUploads[i]);
            } catch (ex) {
                try {
                    this.uploadSvc.savePhotoUpload($(this.mUploads[i].id));
                }
                catch(e) {
                    dump(ex + '\n');
                }
            }
        }
    },
      
    synch: function () {
        if(!this.mUploads || this.mUploads.length==0) return;
        if(this.mDisplayMode == MODE_DISPLAY_MULTI) {
            //var shouldSave = false;
            for (var i=0; i<this.mUploads.length; i++) {
                // encapsulating this is try catch, for sometimes the dom nodes
                // don't exist!
                try {
                    if ($("title_change").checked) {
                        var photoTitle;
                        photoTitle = ($("title").value.length == 0) ? this.mUploads[i].fileName : $("title").value;
                        this.mUploads[i].setAttribute("title", photoTitle);
                        
                        //shouldSave = true;
                    }
                    if ($("description_change").checked) {
                        this.mUploads[i].setAttribute("description", $("description").value);
                        //shouldSave = true;
                    }

                    if ($("tags_change").checked) {
                        this.mUploads[i].setAttribute("tags", $("tags").value);
                    }
                    if ($("permissions_change").checked) {
                        this.mUploads[i].setAttribute("is_public", $("is_public").value=="public");
                        this.mUploads[i].setAttribute("is_friend", $("is_friend").checked);
                        this.mUploads[i].setAttribute("is_family", $("is_family").checked);
                        //shouldSave = true;
                    }
                } catch(e) {
                }
            }
            //if (shouldSave) this.save(this.mUploads[i]);
        } else {
            if(this.mOriginalData.hasChanged("title")) this.mUploads[0].title =  $("title").value;
            if (this.mOriginalData.hasChanged("filename") && $("filename").value.length) this.mUploads[0].fileName = $("filename").value;
            if(this.mOriginalData.hasChanged("description")) this.mUploads[0].description = $("description").value;
            if(this.mOriginalData.hasChanged("tags")) this.mUploads[0].tags = $("tags").value;
            if(this.mOriginalData.hasChanged("is_public")) this.mUploads[0].is_public = $("is_public").value=="public";
            if(this.mOriginalData.hasChanged("is_friend")) this.mUploads[0].is_friend = $("is_friend").checked;
            if(this.mOriginalData.hasChanged("is_family")) this.mUploads[0].is_family = $("is_family").checked;
            //this.save(this.mUploads);
        }

        flock.tags.addTagElementsToFormHistory($("tags").pills);
        this.save();
    },
    
    updateCommands: function() {
         switch (this.mDisplayMode) {
            case MODE_DISPLAY_SINGLE:
                if ($('image').hasCrop)
                    $('cmdCrop').removeAttribute('disabled');
                else
                     $('cmdCrop').setAttribute('disabled', true);
                    
                $('cmdRotateLeft').removeAttribute('disabled');
                $('cmdRotateRight').removeAttribute('disabled');
                $('cmdRevert').removeAttribute('disabled');
                if(this.mUploads[0].crop=="0,0,1,1" && this.mUploads[0].rotation=="0") {
                    $('cmdRevert').setAttribute('disabled', true);
                }
                else {
                    $('cmdRevert').removeAttribute('disabled');
                }

                $("title_change").setAttribute('collapsed', true);
                $("description_change").setAttribute('collapsed', true);
                $("tags_change").setAttribute('collapsed', true);
                $("permissions_change").setAttribute('collapsed', true);
                
                if (this.mUploadType != UPLOAD_TYPE_DND)
                    $('flock-photoprops-disable').setAttribute('hidden', true);
            break;
            
            case MODE_DISPLAY_MULTI:
                $('cmdCrop').setAttribute('disabled', true);
                //$('cmdRevert').setAttribute('disabled', true);
                if (this.mUploadType != UPLOAD_TYPE_DND)
                    $('flock-photoprops-disable').setAttribute('hidden', true);
            break;
            
            case MODE_DISPLAY_NO_PHOTOS:
                //$('photoPropsCmds').setAttribute('disabled', true);
                $('cmdRotateLeft').setAttribute('disabled', true);
                $('cmdRotateRight').setAttribute('disabled', true);
                $('cmdCrop').setAttribute('disabled', true);
                $('cmdRevert').setAttribute('disabled', true);
                $('flock-photoprops-disable').removeAttribute('hidden');
            break;
            default:
            break;
         }
         switch (this.mDisplayMode) {
            case MODE_DISPLAY_NO_PHOTOS:
            case MODE_DISPLAY_SINGLE:
                $("title_change").setAttribute('collapsed', true);
                $("description_change").setAttribute('collapsed', true);
                $("tags_change").setAttribute('collapsed', true);
                $("permissions_change").setAttribute('collapsed', true);
            break;
            case MODE_DISPLAY_MULTI:
                $("title_change").setAttribute('collapsed', false);
                $("description_change").setAttribute('collapsed', false);
                $("tags_change").setAttribute('collapsed', false);
                $("permissions_change").setAttribute('collapsed', false);
            break;
         }

         
         /*if (this.mIsDirty) {
            $('cmdRevert').removeAttribute('disabled');
         } else {
            $('cmdRevert').setAttribute('disabled', true);
         }*/
    },
    
    setupUIFields: function(aApiShortname) {
        if (!aApiShortname)
            return;
        this.mApiMgr = Components.classes['@flock.com/photo-api-manager;1?']
                                .getService(Components.interfaces.flockIPhotoAPIManager);
                                
        var api = this.mApiMgr.getAPIFromShortname(aApiShortname);
         $('photo-title').hidden = !api.supportsFeature('title');
         $('photo-filename').hidden = !api.supportsFeature('fileName');
         $('photo-tags').hidden = !api.supportsFeature('tags');
         $('photo-permissions').hidden = !api.supportsFeature('privacy');
    },

}


/*********************
 * Photo data store
 */
OriginalData.prototype.hasChanged = function(aID) {
    if($(aID).nodeName=="checkbox") {
        if(this[aID]==$(aID).checked) return false;
    }
    else {
        if(this[aID]==$(aID).value) return false;
    }
    //dump("has changed " + aID + " " + this[aID] + " " + $(aID).value + "\n");
    return true;
}
OriginalData.prototype.remember = function() {
    this.title = $('title').value;
    this.filename = $('filename').value
    this.tags = $('tags').value;
    this.description = $('description').value;
    this.is_public = $('is_public').value;
    this.is_friend = $('is_friend').checked;
    this.is_family = $('is_family').checked;
}


/*********************
 * Photo rotator object
 */

function PhotoRotater() {
    this.mRotateQ = new Array();
}
PhotoRotater.prototype.mRotateQ = null;
PhotoRotater.prototype.mIsRotating = false;

PhotoRotater.prototype.doRotate = function(aElement, aDegrees) {
    var uploadSvc = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-photo-uploads']
        .getService(Components.interfaces.flockIPhotoUploadService);
    uploadSvc.rotatePhotoUpload(aElement, aDegrees);
    uploadSvc.savePhotoUpload(aElement);
}

PhotoRotater.prototype.add = function(aElement, aDegrees) {
    var uploadObj = {
        element: aElement,
        degrees: aDegrees,
    }
    this.mRotateQ.unshift(uploadObj);

    window.setTimeout(this.startRotateTimer, 0, this);
}

PhotoRotater.prototype.startRotateTimer = function (aInst) {
    if (aInst.mRotateQ.length) {
        var rotateObj = aInst.mRotateQ.pop();
        aInst.doRotate(rotateObj.element, rotateObj.degrees);
        /*
        if(rotateObj.thumbnail) {
            //if (gMode != MODE_DISPLAY_MULTI) {
                rotateObj.thumbnail.setAttribute("thumbnail", rotateObj.element.thumbFileSpec);
                rotateObj.thumbnail.src = rotateObj.element.thumbFileSpec;
                //rotateObj.thumbnail.photoUpdate();
            //}
        }
        */
        window.setTimeout(aInst.startRotateTimer, 200, aInst);
    }
    else {
        this.mIsRotating = false;
        photoProperties.setupPreviewArea();
        
    }
}
