/*
     BEGIN FLOCK GPL
     
     Copyright Flock Inc. 2005-2006
     http://flock.com
     
     This file may be used under the terms of of the
     GNU General Public License Version 2 or later (the "GPL"),
     http://www.gnu.org/licenses/gpl.html
     
     Software distributed under the License is distributed on an "AS IS" basis,
     WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
     for the specific language governing rights and limitations under the
     License.
     
     END FLOCK GPL
*/

const PREF_UPLOADER = 'flock.photo.uploader.';
const INITIAL_FILEPATH = 'initialPath';
const PU_INITIAL = 1;
const PU_SUCCESS = 2;
const PU_HAS_PHOTOS = 3;
const PU_IMPORTING = 4;

var photoUploader = {};
photoUploader.SVC = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-photo-uploads']
    .getService(Components.interfaces.flockIPhotoUploadService);

photoUploader.mState = PU_INITIAL;
photoUploader.mApiMgr = null;
photoUploader.mApiListener = null;
photoUploader.totalNumImportedPhotos = null;

photoUploader.load = function() {
	//window.flock_photoUpload = flock_photoUpload;
    this.SVC.addListener(this.uploadListener);
    $("flock-photo-upload-slideybox").setListener(this.scrolleyboxListener);
    if(photoUploader.getPhotoCount()) {
        $("flock-photo-upload-slideybox").childNodes.item(1).selected = true;
        $("flock-photo-upload-slideybox").childNodes.item(1).focus();
    }
    this.mApiMgr = Components.classes['@flock.com/photo-api-manager;1?']
                            .getService(Components.interfaces.flockIPhotoAPIManager);
    this.mApiListener = {
        onAPIStateChange: function(aApi) {
            flock.photo.util.populateServicesPopup($('servicesMenuList').firstChild);
        },
        onError: function(aError) {
            alert(aError.errorString);
        },
    }
    this.totalNumImportedPhotos = 0;
    this.mApiMgr.addListener(this.mApiListener);
    photoProperties.setupUIFields(this.mApiMgr.getDefaultService().shortName);
    this.updateCommandState();
    //gPhotoAddProcessor = window.opener.gPhotoAddProcessor;
}

photoUploader.unload = function() {
    this.SVC.removeListener(this.uploadListener);
    this.mApiMgr.removeListener(this.mApiListener);
}

/*
photoUploader.uploadAddedListener = {
  observe: function(aSubject, aTopic, aData) {
        dump('>>> got ' + aTopic + '\n\n');
   switch(aTopic) {
    case "flock:photouploader:batchUploadAddStarted":
      photoUploader.totalNumImportedPhotos = aData;
      $('import-current-number').setAttribute('value', 0);
      $('import-total-number').setAttribute('value', aData);
      $('import-progressmeter').removeAttribute('value');
      photoUploader.mState = PU_IMPORTING;
      photoUploader.updateCommandState();
    break;
    
    case "flock:photouploader:uploadAdded":
      //var photo = document.getElementById(aData);
      //if (gPhotoAddProcessor)
      var importedPhotos = window.opener.gPhotoAddProcessor.mIndex;
      var progress = ((importedPhotos-1)/photoUploader.totalNumImportedPhotos)*100;
      $('import-progressmeter').setAttribute('value', progress + '%');
    break;
    
    case "flock:photouploader:uploadAddStarted":
      var importedPhotos = window.opener.gPhotoAddProcessor.mIndex;
      var sourceFile = flock_getFileFromURL(aData);
      var title = sourceFile.leafName;
      $('import-current-photo-title').setAttribute('value', title);
      //$('import-current-photo').setAttribute('src', aData);
      $('import-current-number').setAttribute('value', importedPhotos);
      photoUploader.mState = PU_IMPORTING;
      photoUploader.updateCommandState();
    break;
    
    case 'flock:photouploader:batchUploadAddComplete':
      if(photoUploader.getPhotoCount()) {
        photoUploader.mState = PU_HAS_PHOTOS;
        photoUploader.enableSlideybox();
        $("flock-photo-upload-slideybox").childNodes.item(1).selected = true;
        $("flock-photo-upload-slideybox").childNodes.item(1).focus();
      } else {
        photoUploader.mState = PU_INITIAL;
      }
      photoUploader.updateCommandState();
    break;
    }
  },
}
*/
photoUploader.onClose = function() {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                             getService(Components.interfaces.nsIPromptService);
    var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
    var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());
    
    var dialogMessage = "";
    var titleMessage = "";
    
    if (this.SVC.isUploading) {
        titleMessage = stringBundle.GetStringFromName("flock.photo.upload.er.title");
        dialogMessage = stringBundle.GetStringFromName("flock.photo.upload.er.msg");

        var rv = promptService.confirm(null, titleMessage, dialogMessage);
        if (rv) {
            this.SVC.cancelUpload();
            return true;
        } else {
            return false;
        }
    } else if (this.SVC.isImporting) {
       titleMessage = stringBundle.GetStringFromName("flock.photo.upload.er.title");
       dialogMessage = stringBundle.GetStringFromName("flock.photo.import.er.msg");
       var rv = promptService.confirm(null, titleMessage, dialogMessage);
        if (rv) {
            this.SVC.cancelImport();
            return true;
        } else {
            return false;
        }
       
    } else 
      return true;
}

photoUploader.cancelUpload = function() {
    this.mRecentlyCancelled = true;
    this.SVC.cancelUpload();
}

photoUploader.updateCommandState = function(aDisable) {
    //first, see if we have uploads
    if (this.mState == PU_IMPORTING) {
      $('flock-photo-import-deck').selectedIndex = 1;
      $("flock-photo-upload-actions").setAttribute("hidden", true);
      $("flock-photo-cancel-actions").setAttribute("hidden", true);
      $("cmd_removePhoto").setAttribute("disabled", true);
      $("cmd_doUpload").setAttribute("disabled", true);
      $("cmd_addFiles").setAttribute("disabled", true);
      $("cmd_launchSettings").setAttribute("disabled", true);
      photoUploader.disableSlideybox();
      this.updateMessage();
      return;
     } else {
       $('flock-photo-import-deck').selectedIndex = 0;
    }
    
    if(photoUploader.SVC.isUploading || aDisable) {
        photoUploader.disableSlideybox();
        photoProperties.clearSelection();
        $("flock-photo-upload-actions").setAttribute("hidden", true);
        $("flock-photo-cancel-actions").removeAttribute("hidden");
        $("cmd_removePhoto").setAttribute("disabled", true);
        $("cmd_doUpload").setAttribute("disabled", true);
        $("cmd_addFiles").setAttribute("disabled", true);
        $("cmd_launchSettings").setAttribute("disabled", true);
        return;
    }
    
    
    $("flock-photo-upload-actions").removeAttribute("hidden");
    $("flock-photo-cancel-actions").setAttribute("hidden", true);

    $("cmd_addFiles").removeAttribute("disabled");
    $("cmd_launchSettings").removeAttribute("disabled");

    var count = this.getPhotoCount();
    var selectionCount = this.getSelectionCount();

    if(count==0) {
        // disable props, upload
        // setup message
        this.mState = PU_INITIAL;
        $("cmd_doUpload").setAttribute("disabled", true);
    }
    else {
        this.mState = PU_HAS_PHOTOS;
        $("cmd_doUpload").removeAttribute("disabled");
    }

    if(selectionCount==0) {
        $("cmd_removePhoto").setAttribute("disabled", true);
    }
    else {
        $("cmd_removePhoto").removeAttribute("disabled");
    }
    this.updateMessage();
}

photoUploader.updateMessage = function(aOverride) {
    if(aOverride) {
        this.mState = aOverride;
    }
    var message = "";
    switch(this.mState) {
        case PU_INITIAL:
            $("flock-photo-instructions").removeAttribute("hidden");
            message = $("flock-photo-instructions-message").getAttribute("msgInitial");
        break;
        case PU_SUCCESS:
            $("flock-photo-instructions").removeAttribute("hidden");
            message = $("flock-photo-instructions-message").getAttribute("msgSuccess");
        break;
        case PU_HAS_PHOTOS:
            $("flock-photo-instructions").setAttribute("hidden", true);
            message = "";
        break;
        
        case PU_IMPORTING:
            $("flock-photo-instructions").setAttribute("hidden", true);
            message = "";
        break;
    }
    $("flock-photo-instructions-message").value = message;
}

photoUploader.getPhotoCount = function() {
    return ($("flock-photo-upload-slideybox").childNodes.length - 1);
}

photoUploader.getSelectionCount = function() {
    var selection = $("flock-photo-upload-slideybox").getSelection();
    return selection.length;
}

photoUploader.getAllPhotoUploads = function() {
    var photouploads = document.getElementsByTagName('photoupload');
    var uploadsArray = Array();
    for (var i=1; i < photouploads.length; i++) {
        uploadsArray.push(photouploads[i]);
    }
    return uploadsArray;
}

photoUploader.addPhotoFiles = function() {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                  .getService(Components.interfaces.nsIPrefService)
                  .getBranch(PREF_UPLOADER);

    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var init_dir = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    var init_path_pref = getUnicodePref(INITIAL_FILEPATH, prefs);
    var photoUrls = Array();

    if (!init_path_pref) {
        var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                 .getService(Components.interfaces.nsIProperties);
                 
// platform specificness                 
//@line 277 "/cygdrive/K/tinderbuild/src/flock/mozilla/browser/components/flock/photo/content/photoUploader.js"
        init_path_pref = dirService.get("DeskV", Components.interfaces.nsIFile).parent.path;
        init_path_pref = init_path_pref + "\\My Pictures";
//@line 282 "/cygdrive/K/tinderbuild/src/flock/mozilla/browser/components/flock/photo/content/photoUploader.js"

        init_dir.initWithPath(init_path_pref);
        
        if (init_dir.path && init_dir.path.length)
            init_path_pref = init_dir.path;
        
        setUnicodePref(INITIAL_FILEPATH, prefs, init_path_pref);
        
    } else {
        init_dir.initWithPath(init_path_pref);
    }

    var fp = Components.classes["@mozilla.org/filepicker;1"]
    	     .createInstance(nsIFilePicker);

    var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
    var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

    fp.init(window, stringBundle.GetStringFromName("flock.photo.upload.add"), nsIFilePicker.modeOpenMultiple);

    fp.appendFilters(nsIFilePicker.filterAll);
  
    var homeDir = null;
    var mypicsPath = null;

    fp.displayDirectory = init_dir;
      
    if ( fp.show( ) == nsIFilePicker.returnOK && fp.files ) {
        var enum = fp.files;
        var item = null;
        var file = null;
        var path = null;  
        while (enum.hasMoreElements()) {
            item = enum.getNext();
            file = item.QueryInterface(Components.interfaces.nsIFile);
            path = file.path.toLowerCase();
            if (path.match(/.+jpg$/) ||
                 path.match(/.+jpeg$/)||
                 path.match(/.+bmp$/)||
                 path.toLowerCase().match(/.+png$/) ||
                 path.toLowerCase().match(/.+gif$/)) 
                {
                    //flock.photo.util.handlePhotoDrop(flock_getURLFromFile(file), null, false);   
                    photoUrls.push(flock_getURLFromFile(file));
               }
       }       
        //flock.photo.util.processPhotoImportQueue();
        window.opener.gPhotoAddProcessor = new PhotoDropProcessor(photoUrls, null); 
        window.opener.gPhotoAddProcessor.start();
        var lastSelectedDir  = file.path.substring(0, file.path.indexOf(file.leafName));
        setUnicodePref(INITIAL_FILEPATH, prefs, lastSelectedDir);
        $('flock-photo-instructions').setAttribute('hidden', true);
    }
}
       

photoUploader.enableSlideybox = function() {
    $("flock-photo-disable").setAttribute("hidden", true);
}

photoUploader.disableSlideybox = function() {
    $("flock-photo-disable").removeAttribute("hidden");
}

// this may move to slideybox
photoUploader.selectFirstPhoto = function() {
    $("flock-photo-upload-slideybox").firstChild.nextSibling.selected = true;
    $("flock-photo-upload-slideybox").firstChild.nextSibling.focus();
}

photoUploader.uploadListener = {
    mErrorOnUpload: false,
    mWindow: window,
    importBatchSize: 0,
    onUploadStart: function (aCurrentUpload, aCount, aMax) {
        // clear properties selection
        photoUploader.mRecentlyCancelled = false;
        photoUploader.uploadListener.mErrorOnUpload = false;
        photoUploader.updateCommandState();

        var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
        var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
        var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

        var msg = stringBundle.GetStringFromName("flock.photo.upload.progress").replace(/%count/, aCount).replace(/%max/, aMax).replace(/%title/,aCurrentUpload.title);

       // -1 because the current photo is not uploaded yet
        var percent = ((aCount - 1)*100) / aMax;
        $('uploading-title').setAttribute("value", msg);
        $('upload-progressmeter').setAttribute("value", percent.toString()+"%");
    },
    onUpload: function (aCurrentUpload, aCount, aMax) {
    },
    onUploadAdd: function (aCurrentUpload) {
        if(!aCurrentUpload) {
            photoProperties.synch();
            photoProperties.invalidate();
        }
        else {
            if(photoProperties.updateSelectedItem(aCurrentUpload)) {
                $(aCurrentUpload.id).parentNode.selected = true;
            }
            photoUploader.updateCommandState();
            if(photoUploader.getPhotoCount()==1) {
                photoUploader.selectFirstPhoto();
            }
        }
    },
    onUploadRemove: function () {
        if(photoUploader.SVC.isUploading) return;
        photoUploader.updateCommandState();
    },
    onUploadComplete: function () {
        $('flock-photo-upload-deck').selectedIndex=0;
        photoUploader.enableSlideybox();
        photoUploader.updateCommandState();
        // Delete the temporary directory, unless the single photo uploader is open
        
        if (!this.mErrorOnUpload && !photoUploader.mRecentlyCancelled)
        {
            var window = this.mWindow;
            var showUploadedPics = flock_getBoolPref('flock.photo.uploader.showPhotosOnUpload');
            if (showUploadedPics) {
                var apimgr = Components.classes['@flock.com/photo-api-manager;1?'] 
                                  .getService(Components.interfaces.flockIPhotoAPIManager);
                var authPerson = apimgr.getDefaultService().getAuthPerson();
                var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                          .getService(Components.interfaces.nsIWindowMediator);
                var win = wm.getMostRecentWindow('navigator:browser');
                
                if (win) {
                    win.flock_topbarSelect(win.document.getElementById('flock_PhotoBrowseTopbarBroadcaster'), true);
                    if (win.TopbarJSCtx().photoBrowser) win.TopbarJSCtx().photoBrowser.loadPersonPre(authPerson);
                }
            }
            var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
              .getService(Components.interfaces.nsIWindowMediator);
            var singleUploadWindow = false;
            var allWindows = windowMediator.getEnumerator(null);
            while(allWindows.hasMoreElements()) {
              var window = allWindows.getNext();
              if (window.name == "singlePhotoUploading")
                singleUploadWindow = true;
            }
            if (!singleUploadWindow)
              window.setTimeout("photoUploader.SVC.cleanPhotoUploadCache();", 1000);
            window.setTimeout("photoUploader.updateMessage(PU_SUCCESS);", 0);//force override
        }
    },
    onUploadError: function (aUpload, aCount, aMax, aError) {
        var errorMessage = aError.errorString;
        // if it's an unknow error display any addition info the server might return
        if (aError.errorCode == aError.PHOTOSERVICE_UNKNOWN_ERROR)
            errorMessage += "\n\nAdditionally, the server returned the message " + aError.serviceErrorString;
        alert(aError.errorString);
        photoUploader.enableSlideybox();
        this.mErrorOnUpload = true;
    },
    onAuthStateChange: function (aState) {
    },
    onMediaImportError: function() {
    
    },
    onUploadCreateError: function(aFile) {
        var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
        var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
        var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

        alert(stringBundle.GetStringFromName("flock.photo.upload.er.bad").replace('%1', aFile));
        photoUploader.updateCommandState();
    },
    
    onMediaBatchImportStart: function(aBatchSizeNum) {
      this.importBatchSize = aBatchSizeNum;
      $('import-current-number').setAttribute('value', 0);
      $('import-total-number').setAttribute('value', aBatchSizeNum);
      $('import-progressmeter').removeAttribute('value');
      photoUploader.mState = PU_IMPORTING;
      photoUploader.updateCommandState();
    },
    
    onMediaBatchImportFinish: function() {
        photoUploader.mState = PU_INITIAL;
        photoUploader.enableSlideybox();
        var firstPhoto = $("flock-photo-upload-slideybox").childNodes.item(1);
        if (firstPhoto) {
          firstPhoto.selected = true;
          firstPhoto.focus();
        }
        photoUploader.updateCommandState();
    },
    
    onMediaImportFinish: function(aMediaUpload, aCount) {
      var progress = (aCount/this.importBatchSize)*100;
      $('import-progressmeter').setAttribute('value', progress + '%');
    },
    
    onMediaImportStart: function (aFilename, aNumInBatch, aBatchSizeNum) {
      this.importBatchSize = aBatchSizeNum;
      $('import-total-number').setAttribute('value', aBatchSizeNum);
      $('import-current-photo-title').setAttribute('value', aFilename);
      $('import-current-number').setAttribute('value', aNumInBatch);
      photoUploader.mState = PU_IMPORTING;
      photoUploader.updateCommandState();
    },
}

photoUploader.scrolleyboxListener = {
    onDrop: function(aHoverChild, aSelection) {
        try {
            photoProperties.synch();
            var svc = photoUploader.SVC;
            var hoverChild = aHoverChild;
            if(hoverChild) hoverChild = svc.getPhotoUpload(hoverChild.firstChild.id);
            var ids = new Array();
            for(var i=0;i<aSelection.length;++i) {
                ids[i] = aSelection[i].firstChild.id;
            }
            var hoverID = null;
            if(aHoverChild) hoverID = aHoverChild.firstChild.id;

            for(var i=0;i<ids.length;++i) {
                if(hoverID && hoverID==ids[i]) continue;
                svc.addPhotoUpload(hoverChild, svc.getPhotoUpload(ids[i]));
            }
            for(var i=0;i<ids.length;++i) {
                $(ids[i]).parentNode.selected = true;
            }
        }
        catch(e) {
            alert(e);
        }
    },
    onScrollRight: function() {
    },
    onScrollLeft: function() {
    },
    onSelect: function(aSelection) {
        if(photoUploader.SVC.isUploading) return;
        photoUploader.updateCommandState();
        var uploadItems = Array();
        for (var i=0; i < aSelection.length; i++)
            uploadItems.push(aSelection[i].firstChild)
        
         photoProperties.setSelection(uploadItems);
        // do the selection
    },
    onRemoveSelection: function(aSelection) {
        // do the clear selection

        //remove from service
        var svc = photoUploader.SVC;
        photoUploader.removeSelection(aSelection);
        
        
        //update state
    }
}

photoUploader.cancelImport = function() {
  this.SVC.cancelImport();
}
photoUploader.doUpload = function() {
    var params = {};
    params.photoUploads = this.getAllPhotoUploads();
    var rv = flock.photo.util.launchUploadOptions(params);

    if (params.launchUploader) {
        $('flock-photo-upload-deck').selectedIndex=1;
    }
    // set the album id on the photo uploads
    // this should probably be an attribute on the service
    // not each particular photo upload object
    var uploads = this.getAllPhotoUploads();
    for (var i=0; i < uploads.length; i++) {
        uploads[i].album = params.albumId;
        this.SVC.savePhotoUpload(uploads[i]);
    }
    
    if (params.apiName) {
        window.setTimeout(this.reallyDoUpload, 0, params.apiName);
        this.updateCommandState(true);
//        this.reallyDoUpload(params.apiName);
    }
}

photoUploader.reallyDoUpload = function(aApiShortname) {
    var api = photoUploader.mApiMgr.getAPIFromShortname(aApiShortname);
    photoUploader.mApiMgr.setDefaultService(aApiShortname);
    photoUploader.SVC.upload(api);
}

photoUploader.removeSelection = function(aSelection) {
    if (!aSelection) {
        aSelection = $("flock-photo-upload-slideybox").getSelection();
    }
    
    photoProperties.clearSelection();
    for (var i=0; i < aSelection.length; i++) {
        this.SVC.removePhotoUpload(aSelection[i].firstChild);
    }
    
}

photoUploader.startUploadFlow = function() {
    var inst = this;
    var service = this.mApiMgr.getDefaultService();
    if (service) {
        if(service.isLoggedIn)
            this.doUpload();
        else {
            var loginListener = {
                onAuth: function() {
                    if (service.isLoggedIn) {
                        inst.doUpload();
                    } 
                },
                
                onError: function(aError) {
                    alert(aError.errorString);
                },
            }
           var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
           var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
           var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

           var message =  stringBundle.GetStringFromName("flock.photo.upload.er.open").replace(/%1/g, service.serviceName);
           message += "\n";
           message += stringBundle.GetStringFromName("flock.photo.upload.er.msg2").replace('%1', service.serviceName);
           alert(message);
           
           // there is a bug in the apimanager which causes multiple state changes
           // which prevents us to use it -- for now call the login on the photoservice directly
           service.login(loginListener);
      }
    } else {
        var params = {};
        params.hasValidSvc = false;
        rv = flock.photo.util.launchSettings(params);
        if (params.hasValidSvc)
            this.doUpload();
    }
}

function getUnicodePref(prefName,prefBranch) {
  if (prefBranch.prefHasUserValue(prefName))
    return prefBranch.getComplexValue(prefName, Components.interfaces.nsISupportsString).data;
  return null;
}

function setUnicodePref(prefName, prefBranch, prefValue) {
    var sString = Components.classes["@mozilla.org/supports-string;1"].
                    createInstance(Components.interfaces.nsISupportsString);
    sString.data = prefValue;
    prefBranch.setComplexValue(prefName, Components.interfaces.nsISupportsString, sString);
}

